#ifndef EXTRA_KEYSYM
#define EXTRA_KEYSYM

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: extra_keysym.h,v 1.1.1.1 1998/07/16 19:29:29 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

/*
 * Floating Accent
 */

#define	Sun_XK_FA_Grave		0x1005FF00
#define	Sun_XK_FA_Circum	0x1005FF01
#define	Sun_XK_FA_Tilde		0x1005FF02
#define Sun_XK_FA_Acute         0x1005FF03
#define Sun_XK_FA_Diaeresis     0x1005FF04
#define Sun_XK_FA_Cedilla       0x1005FF05
 
/*
 * Miscellaneous Functions
 */

#define	Sun_XK_F36		0x1005FF10	/* Labeled F11 */
#define	Sun_XK_F37		0x1005FF11	/* Labeled F12 */

#define Sun_XK_Sys_Req   	0x1005FF60
#define Sun_XK_Print_Screen	0x0000FF61	/* Same as XK_Print */

/*
 * International & Multi-Key Character Composition
 */

#define Sun_XK_Compose		0x0000FF20	/* Same as XK_Multi_key */
#define Sun_XK_AltGraph		0x0000FF7E	/* Same as XK_Mode_switch */

/*
 * Cursor Control
 */

#define Sun_XK_PageUp		0x0000FF55 	/* Same as XK_Prior */
#define Sun_XK_PageDown		0x0000FF56	/* Same as XK_Next */

/*
 * Open Look Functions
 */

#define Sun_XK_Undo		0x0000FF65	/* Same as XK_Undo */
#define Sun_XK_Again		0x0000FF66	/* Same as XK_Redo */
#define Sun_XK_Find		0x0000FF68	/* Same as XK_Find */
#define Sun_XK_Stop		0x0000FF69	/* Same as XK_Cancel */
#define Sun_XK_Props		0x1005FF70
#define Sun_XK_Front		0x1005FF71
#define Sun_XK_Copy		0x1005FF72
#define Sun_XK_Open		0x1005FF73
#define Sun_XK_Paste		0x1005FF74
#define Sun_XK_Cut		0x1005FF75

 /* 
  * Keysyms for Hardware control keys on Sun Type5 keyboard
  * Note: No embedded "XK_" strings in keysym defines.
  * Other keysyms should be changed to do this as well
  */

#define	Sun_PowerSwitch		0x1005FF76
#define Sun_AudioLowerVolume	0x1005FF77
#define Sun_AudioMute		0x1005FF78
#define Sun_AudioRaiseVolume	0x1005FF79
#define	Sun_VideoDegauss		0x1005FF7A
#define	Sun_VideoLowerBrightness 	0x1005FF7B
#define Sun_VideoRaiseBrightness 	0x1005FF7C
#define	Sun_PowerSwitchShift	0x1005FF7D

#endif

