/*-----------------------------------------------------------------------
 * Xclasses code file
 * ($Id: chartable.h,v 1.1 1998/08/08 18:48:03 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Ralf Becker
 *
 * HTML Parser system
 * (c) 1998 by Jürgen Schmitz - all rights reserved
 * THIS IS NOT GPL OR SOMETHING OTHER FREE LICENSE!!
 * it may only be used together with Xclasses!
 *-----------------------------------------------------------------------
 */

#ifndef INCLUDE_CHARTABLE_H
#define INCLUDE_CHARTABLE_H

#define COUNT	101
#define NBSP	160

class CharTable {
	private:
		struct TranslateCode {
			char *ascii;
			unsigned char code;
		};
		static TranslateCode translate[COUNT];

	public:
		static int Translate(char *);
};

#endif
