#ifndef CODE_APPWINDOWP_H
#define CODE_APPWINDOWP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: appwindow_p.h,v 1.1.1.1 1998/07/16 19:29:32 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/group.h>
#include <Xclasses/menu.h>
#include <Xclasses/appwindow.h>
#include <Xclasses/dlist.h>


//#define EXTRABAR	baron

#ifdef EXTRABAR
static int baron=1;
#endif

class pappwindow
{
	friend class appwindow;

public:

	class appwindow *o;

	node	winnode;

	int 	x,y,w,h,ps;
	int		ad,ad_x,ad_y,ad_w,ad_h;
	int		mtop;
	int		closeme:1,
			extra_right:1,
			showextra:1,
			blocked:1;

	class callback cb;

	menu	*amenu;
	group	*gruppe;
	group	*extragruppe;

#ifdef EXTRABAR
	appwindow *menubar;
	group     *menugroup;
#endif

	pappwindow();
	~pappwindow();

	static void alarmhandler(void);
	static void alarmhandler2(void);
	static void unalarm(void);
};

#endif
