#ifndef CODE_XWINDOWSP_H
#define CODE_XWINDOWSP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: Xwindows_p.h,v 1.2 1998/11/08 23:42:49 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/Xwindows.h>
#include <Xclasses/dlist.h>


class pXwindows
{
	friend class Xwindows;

public:
	class Xwindows *o;

	int			flags;
	int			noWM:1,
				shadowmode:1,
				atpointer:1,
				free:1;

	node		winnode;
	
	XSizeHints	sizehints;
	XWMHints	wmhints;
	XClassHint	classhint;
	Atom		protocol[2];
	Window		win;
	Cursor		cursor;
	int			bargc;
	char 		**bargv;
	char		*title;
	char		*ititle;
	keyclass	*keys;

	gadget		*keyactive;


	pXwindows();
	~pXwindows();

	void DefaultHints(void);
	void FreeHints(void);
	void ReadRes(int &x,int &y,int &w,int &h);
};

#endif
