#ifndef INCL_XCPROGRAM_H
#define INCL_XCPROGRAM_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/types.h>
#include <Xclasses/Xcolors.h>

class XCprogram : public Xcolors
{
	friend class pXCprogram;
	class pXCprogram *p;
public:

	XCprogram(char *, ...);
    virtual ~XCprogram();
};

#define XCP_Done				0x00000000
#define XCP_Author				0x80000001
#define XCP_Copyright			0x80000002
#define XCP_Version				0x80000003
#define XCP_Description			0x80000004
#define XCP_Desc				XCP_Description
#define XCP_Address				0x80000010
#define XCP_EMail				0x80000011
#define XCP_WWW					0x80000012
#define XCP_OwnText				0x80000020
#define XCP_OwnHTML				0x80000021
#define XCP_XC_logo				0x80000030	// obsolete, use XCP_Xclasses_logo
#define XCP_gfxobject			0x80000031
#define XCP_Resources			0x80000032
#define XCP_Xclasses_logo		0x80000033
#define XCP_RequiredXclasses	0x80000034

#define XCP_MultiSortByJob		0x80000040
#define XCP_MultiAuthor			0x80000041
#define XCP_MultiAuthorJob		0x80000042
#define XCP_MultiAuthorEMail	0x80000043

#endif
