#ifndef INCL_STRBUF_H
#define INCL_STRBUF_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: strbuf.h,v 1.5 1998/08/14 20:57:58 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

class Xcl_strbuf
{
	char *buf;
	unsigned int used;
	unsigned int size;
public:
	Xcl_strbuf(void);
	~Xcl_strbuf();
	char *operator+(char *);
	void operator+=(char *);
	char *operator()(void);
	void clear(void);
	void replace(char *);

	void operator()(int size);	// change buffersize (eg: object(GetSize()+1); GetCopy(object(),GetSize()+1); object.update(); )
	void update(void);			// update used

	void resize(int r);
};

#endif
