#ifndef CODE_EPRINTF_CXX
#define CODE_EPRINTF_CXX

/*-----------------------------------------------------------------------
 * Xclasses code file
 * ($Id: eprintf.cxx,v 1.1.1.1 1998/07/16 19:29:27 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include "types.h"

void eprintf(char *t)
{
	fprintf(stderr,"Error:\n%s\n",t);
}
void error_printf(char *t,class Xclasses *c)
{
	fprintf(stderr,"Error in class %s:\n%s\n",c->FullName(),t);
}
void warn_printf(char *t,class Xclasses *c)
{
	fprintf(stderr,"Warning in class %s:\n%s\n",c->FullName(),t);
}
void fatal_printf(char *t,class Xclasses *c)
{
	fprintf(stderr,"Fatal in class %s:\n%s\n\n",c->FullName(),t);
}

void outOfMemory(char *classname)
{
	fprintf(stderr,"Fatal error: Out of virtual memory,\nclass %s can't allocate private memory.\nTerminating.\n",classname);
	exit(10);
}

#endif
