#ifndef CODE_RUBBERGROUPP_H
#define CODE_RUBBERGROUPP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: rubbergroup_p.h,v 1.1.1.1 1998/07/16 19:29:29 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/rubbergroup.h>

struct sub
{
	class gadget 	*obj;
	class gadget	*mvgg;
	int				pos;
	int 			width;
	int				minleft;
	int				minright;
	int 			tomin:1,
					tomax:1,
					free:1;
};

#define RULERSIZE	8
#define KNOBSIZE	6
#define KNOBOFFSET	4

class prubbergroup
{
	friend class rubbergroup;

	class rubbergroup *o;

public:
	int asize;
	int aused;

	struct sub *subs;			// array 

	int horiz:1,
		ignorelimit:1,
		moved:1,
		free:1;		

	int rubpos;
	GC rubgc;

	popup pmenu;
	
	prubbergroup();
	~prubbergroup();

	void SetSize(int md);
	void draw(void);
};

#endif
