#ifndef CODE_FILEINPUTP_H
#define CODE_FILEINPUTP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: fileinput_p.h,v 1.2 1998/07/29 23:02:34 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/callable.h>
#include <Xclasses/input.h>
#include <Xclasses/button.h>
#include <Xclasses/go_button.h>
#include <Xclasses/gfx_file.h>
#include <Xclasses/gfx_command.h>

#define INPUTSIZE	1000

class pfileinput : public callable
{
	friend class fileinput;
	class fileinput *o;

public:
	input		fi_input;
	button		*fi_button;
	go_button	*fi_gbutton;
	gfx_object	*fi_gfx;

    pfileinput();
    ~pfileinput();

	void opend(gadget *,int);
	void mode(int m);
};

#endif
