#ifndef CODE_SETUPDIALOGP_H
#define CODE_SETUPDIALOGP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: setup_dialog_p.h,v 1.3 1998/08/06 13:44:01 schmitzj Exp $)
 *
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/callable.h>
#include <Xclasses/menu.h>
#include <Xclasses/appwindow.h>

#define FONTS	8

class psetup_dialog : public callable
{
	friend class setup_dialog;

public:
	class setup_dialog *o;

	appwindow *aw;
	menu	  *hmenu;

    psetup_dialog();
    ~psetup_dialog();

	void readconfhook(void);    
	void writeconfhook(void);
    void readconf(int m=0,char *cnme=NULL);
    void writeconf(int r=0);
	void GUI2config(struct defaultstruct &);
	void config2GUI(struct defaultstruct &);

	void menuhandler(gadget *,int);
	void gghandler(gadget *,int);
	void colorhandler(gadget *,int);
	void font_undo(gadget *g,int i);
	void font_get(gadget *g,int i);
	void close(void);
	void filelisthook(gadget *,int);
};

#endif
