#ifndef CODE_OUTPUTP_H
#define CODE_OUTPUTP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: output_p.h,v 1.4 2000/06/07 18:07:39 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/Xdisplay.h>
#include <Xclasses/output.h>
#include <Xclasses/gadget.h>
#include <Xclasses/Xcolors.h>
#include <Xclasses/Ddraw.h>
#include <Xclasses/go_button.h>
#include <Xclasses/gfx_arrow.h>

class ptoutput: public gadget, private Ddraw
{
	friend class poutput;
public:
    ptoutput() : gadget(), Ddraw(this)
    {};
	~ptoutput() {};

	Window _win(void) { return win; };
	GC _gc(void) { return gc; };
	Window _parent(void) { return parent; };
	int _x(void) { return x; };
	int _y(void) { return y; };
	int _width(void) { return width; };
	int _height(void) { return height; };
	int _active(void) { return active; };
};

class poutput
{
	friend class output;

	class output *o;

public:
	class ptoutput tgg;

    char *text;
    int textwidth;
    char *data;
    int l;

	go_button a1,a2;
	gfx_arrow ar1,ar2;
	int tolong;
	int tolongstart;
	int tolongcount;
	int mx;
    
    int mpos,mend;

	Atom prop[1];
    
    poutput();
    ~poutput();

	void taus(int);
	void cutpaste(XEvent *event);
};

#endif
