#ifndef CODE_COLORNAMEGROUPP_H
#define CODE_COLORNAMEGROUPP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: colornamegroup_p.h,v 1.4 1998/08/06 14:01:36 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/Xdisplay.h>
#include <Xclasses/colornamegroup.h>
#include <Xclasses/listview.h>
#include <Xclasses/dlist.h>
#include <Xclasses/graphic.h>

struct color_name
{
	struct node		base;
	char			name[50];
};

class pcolornamegroup : public callable
{
	friend class colornamegroup;

	class colornamegroup *o;

public:

	listview	list;
	graphic		prev;
	dlist		names;

	char curcolor[50];
	unsigned long acol;

	pcolornamegroup();
	~pcolornamegroup();

	void handle(gadget *,int m);
	void start(void);
};

#endif
