#ifndef CODE_GFXAUTOTEXTP_H
#define CODE_GFXAUTOTEXTP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: gfx_autotext_p.h,v 1.1.1.1 1998/07/16 19:29:31 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/gfx_autotext.h>

class pgfx_autotext
{
	friend class gfx_autotext;

public:
	struct parts
	{
		struct parts *next;
		int 		length;
		int			under;
		int			newline;
		int			width;
		char 		text[1];
	};

	class gfx_autotext *o;

	int 			total_parts;
	int				total_lines;
	int				total_chars;
	parts			*teile;
	int				adjust;
	int				allowunderline:1,
					free:1;

	int X(int w,int cw);
	void UL(gadget *g,int x,int y,parts *p);
	void draw(class gadget *g,int x,int y,int w,int h);
	
	pgfx_autotext(void);
	~pgfx_autotext();
};

#endif
