#ifndef INCL_XCLASSES_H
#define INCL_XCLASSES_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/callable.h>
#include <Xclasses/Xclasses_class.h>
#include <Xclasses/XCprogram.h>

#include <Xclasses/Xdisplay.h>
#include <Xclasses/Xcolors.h>
#include <Xclasses/Xwindows.h>
#include <Xclasses/Xshadow.h>
#include <Xclasses/appwindow.h>

#include <Xclasses/gadget.h>
#include <Xclasses/selgadget.h>
#include <Xclasses/mxgadget.h>

#include <Xclasses/button.h>
#include <Xclasses/fbutton.h>
#include <Xclasses/go_button.h>
#include <Xclasses/selbutton.h>
#include <Xclasses/go_selbutton.h>
#include <Xclasses/mxbutton.h>
#include <Xclasses/choice.h>
#include <Xclasses/input.h>
#include <Xclasses/multilistview.h>
#include <Xclasses/listview.h>
#include <Xclasses/treeview.h>
#include <Xclasses/dirview.h>
#include <Xclasses/scroller.h>
#include <Xclasses/slider.h>
#include <Xclasses/selknob.h>
#include <Xclasses/mxknob.h>
#include <Xclasses/checkbox.h>
#include <Xclasses/popbutton.h>

#include <Xclasses/textbox.h>
#include <Xclasses/autotextbox.h>
#include <Xclasses/htmltextbox.h>
#include <Xclasses/indicator.h>
#include <Xclasses/xclock.h>
#include <Xclasses/output.h>
#include <Xclasses/text.h>
#include <Xclasses/ruler.h>
#include <Xclasses/graphic.h>
#include <Xclasses/anim.h>
#include <Xclasses/image.h>
#include <Xclasses/html_gadget.h>

#include <Xclasses/group.h>
#include <Xclasses/bordergroup.h>
#include <Xclasses/spacegroup.h>

#include <Xclasses/lefttext.h>
#include <Xclasses/visible.h>
#include <Xclasses/card.h>
#include <Xclasses/rubbergroup.h>
#include <Xclasses/listgroup.h>
#include <Xclasses/fileinput.h>
#include <Xclasses/fontgroup.h>
#include <Xclasses/colornamegroup.h>

#include <Xclasses/menu.h>
#include <Xclasses/popup.h>
#include <Xclasses/gpopup.h>

#include <Xclasses/easy_dialog.h>
#include <Xclasses/file_dialog.h>

#include <Xclasses/keyclass.h>
#include <Xclasses/dlist.h>
#include <Xclasses/config.h>
#include <Xclasses/tools.h>

#include <Xclasses/gfx_object.h>
#include <Xclasses/gfx_text.h>
#include <Xclasses/gfx_autotext.h>
#include <Xclasses/gfx_locked.h>
#include <Xclasses/gfx_knob.h>
#include <Xclasses/gfx_checkbox.h>
#include <Xclasses/gfx_arrow.h>
#include <Xclasses/gfx_dir.h>
#include <Xclasses/gfx_file.h>
#include <Xclasses/gfx_xpm.h>
#include <Xclasses/gfx_command.h>

#include <Xclasses/message.h>
#include <Xclasses/message_tools.h>

#endif
