#ifndef XCLASSES_CONFIG_P
#define XCLASSES_CONFIG_P

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: config_p.h,v 1.2 2000/01/18 17:12:01 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/config.h>

class pconfig
{
	friend class config;
public:
	class config 		*o;
	char 				*name;
	struct configopts 	*opts;
	dlist 				extralines;
	char 				*dsp;

	unsigned int	leaveunknown:1;

	pconfig();
	~pconfig();

	void printOpt(struct configopts *,FILE *);
	char Opts(char *);
	char *Command(char*);
	char *T2S(char *,char *);
	int S2T(char*,char*);
};


#endif
