#ifndef INCL_RUBBERGROUP_H
#define INCL_RUBBERGROUP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/types.h>
#include <Xclasses/gadget.h>
#include <Xclasses/Ddraw.h>

class rubbergroup : public gadget, private Ddraw
{
	friend class prubbergroup;

	class prubbergroup *p;

public:
	virtual void GExpose(XEvent *);
	virtual int  GActivate(XEvent *event,int why);
	virtual int  GSelected(XEvent *event,int why,int wasabove);
	virtual int  GMouseMove(int mx,int my,int rx,int ry,unsigned int state,int button);
	virtual void GActionPress(int mx,int my,int rx,int ry,unsigned int state,int button,int reason);
	virtual void GActionRelease(int mx,int my,int rx,int ry,unsigned int state,int button,int reason);

    virtual void GSetLimits(void);	// SetSize

	rubbergroup(void);
	virtual ~rubbergroup(void);

	void Add(class gadget *gd);
	void Add(class gadget &gd) { Add(&gd); };

	virtual int Create(void);
    virtual void Free(void);
    virtual void Resize(int width,int height);

	virtual void Lock(void);
	virtual void Unlock(void);
    virtual void Block(int);
    virtual void Unblock(int);

	void Horiz(void);
	void Vert(void);		// Default
};

#endif
