#ifndef CODE_XCLASSESANIM_CXX
#define CODE_XCLASSESANIM_CXX

/*-----------------------------------------------------------------------
 * Xclasses code file
 * ($Id: xclassesanim.cxx,v 1.1.1.1 1998/07/16 19:29:29 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

//#include <Xclasses/xclassesanim.h>
#include "types.h"
#include "../include/xclassesanim.h"

#include "anim/Xclasses_1.xpm"
#include "anim/Xclasses_2.xpm"
#include "anim/Xclasses_3.xpm"
#include "anim/Xclasses_4.xpm"
#include "anim/Xclasses_5.xpm"
#include "anim/Xclasses_6.xpm"
#include "anim/Xclasses_7.xpm"
#include "anim/Xclasses_8.xpm"
#include "anim/Xclasses_9.xpm"
#include "anim/Xclasses_10.xpm"
#include "anim/Xclasses_11.xpm"
#include "anim/Xclasses_12.xpm"
#include "anim/Xclasses_13.xpm"
#include "anim/Xclasses_14.xpm"
#include "anim/Xclasses_15.xpm"
#include "anim/Xclasses_16.xpm"
#include "anim/Xclasses_17.xpm"
#include "anim/Xclasses_18.xpm"
#include "anim/Xclasses_19.xpm"
#include "anim/Xclasses_20.xpm"
#include "anim/Xclasses_21.xpm"
#include "anim/Xclasses_22.xpm"
#include "anim/Xclasses_23.xpm"
#include "anim/Xclasses_24.xpm"
#include "anim/Xclasses_25.xpm"
#include "anim/Xclasses_26.xpm"
#include "anim/Xclasses_27.xpm"
#include "anim/Xclasses_28.xpm"
#include "anim/Xclasses_29.xpm"
#include "anim/Xclasses_30.xpm"
#include "anim/Xclasses_31.xpm"
#include "anim/Xclasses_32.xpm"
#include "anim/Xclasses_33.xpm"
#include "anim/Xclasses_34.xpm"
#include "anim/Xclasses_35.xpm"
#include "anim/Xclasses_36.xpm"
#include "anim/Xclasses_37.xpm"
#include "anim/Xclasses_38.xpm"
#include "anim/Xclasses_39.xpm"
#include "anim/Xclasses_40.xpm"
#include "anim/Xclasses_41.xpm"
#include "anim/Xclasses_42.xpm"
#include "anim/Xclasses_43.xpm"
#include "anim/Xclasses_44.xpm"
#include "anim/Xclasses_45.xpm"
#include "anim/Xclasses_46.xpm"
#include "anim/Xclasses_47.xpm"
#include "anim/Xclasses_48.xpm"
#include "anim/Xclasses_49.xpm"
#include "anim/Xclasses_50.xpm"
#include "anim/Xclasses_51.xpm"

static 
char **pics[]={Xclasses_1,Xclasses_2,Xclasses_3,Xclasses_4,Xclasses_5,
			   Xclasses_6,Xclasses_7,Xclasses_8,Xclasses_9,Xclasses_10,
			   Xclasses_11,Xclasses_12,Xclasses_13,Xclasses_14,Xclasses_15,
			   Xclasses_16,Xclasses_17,Xclasses_18,Xclasses_19,Xclasses_20,
			   Xclasses_21,Xclasses_22,Xclasses_23,Xclasses_24,Xclasses_25,
			   Xclasses_26,Xclasses_27,Xclasses_28,Xclasses_29,Xclasses_30,
			   Xclasses_31,Xclasses_32,Xclasses_33,Xclasses_34,Xclasses_35,
			   Xclasses_36,Xclasses_37,Xclasses_38,Xclasses_39,Xclasses_40,
			   Xclasses_41,Xclasses_42,Xclasses_43,Xclasses_44,Xclasses_45,
			   Xclasses_46,Xclasses_47,Xclasses_48,Xclasses_49,Xclasses_50,
			   Xclasses_51,
			   NULL};

xclassesanim::xclassesanim() : anim()
{
	Name("xclassesanim");
}
xclassesanim::~xclassesanim()
{
}
int xclassesanim::Create(void)
{
	if (!setlimit) Pics(pics);
	Rate(1);
	return anim::Create();
}
void xclassesanim::GSetLimits(void)
{
	Pics(pics);
	anim::GSetLimits();
	setlimit=1;
}
#endif
