#ifndef INCL_INDICATOR_H
#define INCL_INDICATOR_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/types.h>
#include <Xclasses/gadget.h>
#include <Xclasses/Xcolors.h>
#include <Xclasses/Ddraw.h>


class indicator : public gadget, private Ddraw
{
	friend class pindicator;
	
	class pindicator *p;

public:
    virtual void GExpose(XEvent *event);
	virtual int GActivate(XEvent *event,int why);
	virtual void GSetLimits(void);

public:
    indicator(void);
    virtual ~indicator();

	void Title(char *t);
	void VertIndicator(void);	// currently unsupported
	void HorizIndicator(void);	// default
	void CircleIndicator(void);

    int Create(void);

	void MaxValue(int v);
	void MaxValue(int v,int smark);

	void Value(int v);
	void Value(int v,char *t);	// value and text
	void Text(char *t);
};

#endif
