#ifndef CODE_FILEDIALOGP_H
#define CODE_FILEDIALOGP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: file_dialog_p.h,v 1.2 1998/09/23 18:52:20 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/Xdisplay.h>
#include <Xclasses/group.h>
#include <Xclasses/spacegroup.h>
#include <Xclasses/ruler.h>
#include <Xclasses/fbutton.h>
#include <Xclasses/dirview.h>
#include <Xclasses/input.h>
#include <Xclasses/appwindow.h>

class pfile_dialog
{
	friend class file_dialog;

public:
	class file_dialog *o;

	appwindow 	aw;
	group		maingp;
	group		buts;
	dirview		dirs;
	input		pattern;
	input		dirname;
	input		filename;
	ruler		rul;
	fbutton		ok,cancel,gohome,parent;
	char		*oktext,*canceltext,*gohometext,*parenttext;
	char		*patterntext,*dirnametext,*filenametext;

	group		*insertgroup;

	int			save:1,
				append:1,
				dirmode:1,
				free:1;

    pfile_dialog();
    ~pfile_dialog();

    int fileok(Xwindows *);
};

#endif
