#ifndef CODE_SELBUTTONP_H
#define CODE_SELBUTTONP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: selbutton_p.h,v 1.1.1.1 1998/07/16 19:29:28 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/selbutton.h>

class pselbutton
{
	friend class selbutton;

public:
	class selbutton *o;

	int state;
	int key;
	char *text;
	
	pselbutton();
	~pselbutton();
	void Press(void);
	void Release();
};

#endif
