#ifndef INCL_CREATER_H
#define INCL_CREATER_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

class appwindow *Create_appwindow(char *name,char *title,char *icontitle);


class button *Create_button(class Xwindows *,char *name,char key,char *text);
class fbutton *Create_bfutton(class Xwindows *,char *name,char key,char *text);
class checkbox *Create_checkbox(class Xwindows *,char *name,char key,char *text);
class selbutton *Create_selbutton(class Xwindows *,char *name,char key,char *text);
class choice *Create_choice(class Xwindows *,char *name,char key);
class input *Create_input(class Xwindows *,char *name,char key,char *text);
class listview *Create_listview(class Xwindows *,char *name,char key);
class mxknob *Create_mxknob(class Xwindows *,char *name,char key);
class selknob *Create_selknob(class Xwindows *,char *name,char key,char *text);
class mxbutton *Create_mxbutton(class Xwindows *,char *name,char key);
class scroller *Create_scroller(class Xwindows *,char *name,char key);
class slider *Create_slider(class Xwindows *,char *name,char key);
class textbox *Create_textbox(class Xwindows *,char *name,char key);
class dirview *Create_dirview(class Xwindows *,char *name,char key);

class go_button *Create_go_button(class Xwindows *,char *name,char key);
class go_selbutton *Create_go_selbutton(class Xwindows *,char *name,char key);

class visible *Create_visible(class Xwindows *,char *name,char key);

class output *Create_output(class Xwindows *,char *name,char *text);
class ruler *Create_ruler(class Xwindows *,char *name);
class xclock *Create_xclock(class Xwindows *,char *name);
class autotextbox *Create_autotextbox(class Xwindows *,char *name);
class htmltextbox *Create_htmltextbox(class Xwindows *,char *name);
class text *Create_text(class Xwindows *,char *name);
class anim *Create_anim(class Xwindows *,char *name);
class image *Create_image(class Xwindows *,char *name);
class graphic *Create_graphic(class Xwindows *,char *name,int w,int h);
class indicator *Create_indicator(class Xwindows *,char *name);

class card *Create_card(class Xwindows *,char *name);
class rubbergroup *Create_rubbergroup(class Xwindows *,char *name);

class group *Create_group(class Xwindows *,char *name);
class bordergroup *Create_bordergroup(class Xwindows *,char *name,char *text);
class spacegroup *Create_spacegroup(class Xwindows *,char *name,int border);

class popup *Create_popup(class Xwindows *,char *name);
class menu *Create_menu(class Xwindows *,char *name);

#endif
