#ifndef CODE_XCLASSESP_H
#define CODE_XCLASSESP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: Xclasses_p.h,v 1.2 1998/07/30 16:45:18 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#define MAXNAMES	10

class pXclasses
{
	friend class Xclasses;

public:
	class Xclasses 	*o;

	struct node		mynode;
	struct node		nexthash;
	struct node		nexthash2;
	int				hashvalue;
	int				hash2value;
	class Xclasses 	*parentobj;
	char		   	*objname;
	char		   	*pathbuf;
	char			*allnames[MAXNAMES];
	int				alllist;
	int				level;
	int				changed:1;

	pXclasses(void);
	~pXclasses();
	int parentExists(void);
	int checkChange(void);
};

#endif
