#ifndef CODE_VISIBLEP_H
#define CODE_VISIBLEP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: visible_p.h,v 1.1.1.1 1998/07/16 19:29:29 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/Xdisplay.h>
#include <Xclasses/visible.h>
#include <Xclasses/gadget.h>
#include <Xclasses/scroller.h>
#include <Xclasses/slider.h>

#define SCROLLER_HEIGHT	18
#define SCROLLER_WIDTH	18
#define PRAHMEN1		1	// Freiraum aussen
#define PRAHMEN2		1	// Freiraum innen
#define PRAHMENB		1	// Rahmen
#define RAHMEN			(PRAHMEN1+PRAHMEN2+RealSize())
#define PRAHMEN			(PRAHMEN1+PRAHMEN2+o->RealSize())

class pvisible
{
	friend class visible;

	class visible *o;

public:
    class gadget *obj;
    class scroller horiz;
    class scroller vert;
    class slider scale;
    class gadget innen;

	int px;
	int sc;
	int zoom;
	int	h_visible:1,
		v_visible:1,
		free:1;

    pvisible();
    ~pvisible();

	int S_Width(int);
	int S_Height(int);
	void SetProps(void);
	void Move(void);
	void Resize(void);

};

#endif
