#ifndef CODE_GFXOBJECT_CXX
#define CODE_GFXOBJECT_CXX

/*-----------------------------------------------------------------------
 * Xclasses code file
 * ($Id: gfx_object.cxx,v 1.1.1.1 1998/07/16 19:29:31 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include "types.h"
#include <Xclasses/gfx_object.h>
#include <Xclasses/Xclasses_class.h>
#include <Xclasses/dlist.h>

Xclasses basegfxobject("basegfxobject");

gfx_object::gfx_object() : Xcolors()
{
	Name("gfx_object");
	ParentClass(basegfxobject);
}
gfx_object::~gfx_object()
{
}

void gfx_object::Adjust(int a)
{
}

void gfx_object::draw_normal(class gadget *g,int x,int y,int w,int h)
{
}
void gfx_object::draw_selected(class gadget *g,int x,int y,int w,int h)
{
}
void gfx_object::draw_locked(class gadget *g,int x,int y,int w,int h)
{
}
void gfx_object::draw_selectedlocked(class gadget *g,int x,int y,int w,int h)
{
}

int gfx_object::height(class gadget *g)
{
	return NOLIMIT;
}
int gfx_object::width(class gadget *g)
{
	return NOLIMIT;
}

#endif
