#ifndef INCL_TYPES_H
#define INCL_TYPES_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>

#include <Xclasses/Xclasses_class.h>


// State for selgadgets
#define PRESSED		1
#define RELEASED	0

// use this for casting class hooks (like listview::SetHook)
// usage: e.g. Hook(&myclass,(CLASSHOOKCAST)&myclass.function);
#define CLASSHOOKCAST void (*)(...)

// use this for casting gadget callbacks
// usage: e.g. Callback(&myclass,(CALLBACKCAST)&myclass.function);
#define CALLBACKCAST void (*)(gadget*,int)

// creates class with name
#define NEW(classname,objectname)	classname objectname; objectname.Name("objectname");

#endif
