#ifndef INCL_POPBUTTON_H
#define INCL_POPBUTTON_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/types.h>
#include <Xclasses/button.h>

class popbutton : public button 
{
	friend class ppopbutton;

	class ppopbutton *p;

public:
	virtual int GSelected(XEvent *e,int why,int wasabove);

public:
    popbutton();
    virtual ~popbutton();

	int  Create(void);
	void Free(void);
	void Apply(gadget *,char *title=NULL,int size=50);
	void CloseGadget(gadget *);
	void CloseGadget(gadget &g) { CloseGadget(&g); };

    void OpenCallback(void cfunc(void));
	void OpenCallback(callable *cl,void (callable::*gfunc)(void));
};

#define OPENCALLBACK void (callable::*)(void)

#endif
