#ifndef CODE_XCPROGRAMP_H
#define CODE_XCPROGRAMP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: XCprogram_p.h,v 1.1.1.1 1998/07/16 19:29:27 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/gfx_object.h>
#include <Xclasses/XCprogram.h>

#define INTERNALOBJS	"internals"

struct moreauthor
{
	struct moreauthor	*next;
	char				*author;
	char				*what;
	char				*email;
};

class pXCprogram
{
	friend class XCprogram;

public:
	class XCprogram *o;

	class Xclasses *name;

	char		*author;
	char		*desc;
	char		*copyright;
	char		*version;
	char		*address;
	char		*email;
	char		*www;
	char		*owntext;
	char		*ownhtml;
	gfx_object	*gfx;
	moreauthor	*author2;
	int			mauthors;
	char		*resources;
	int			sortbyjob:1,
				free:1;

	pXCprogram(void);
	~pXCprogram();
};

#endif
