#ifndef CODE_GOSELBUTTONP_H
#define CODE_GOSELBUTTONP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: go_selbutton_p.h,v 1.1.1.1 1998/07/16 19:29:28 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/go_selbutton.h>

class pgo_selbutton
{
	friend class go_selbutton;

public:
	class go_selbutton *o;

	int state;
	gfx_object *obj;
	
	int	limit:1,
		glocked:1,
		border:1,
		key:1,
		keylook:1,
		free:1;

    int		minw,minh;
    int		maxw,maxh;
    int		txtheight;

	pgo_selbutton();
	~pgo_selbutton();
	void Press(void);
	void Release();
};

#endif
