#ifndef HTML_GADGET_H
#define HTML_GADGET_H

#include <Xclasses/gadget.h>
#include <Xclasses/Ddraw.h>

class html_gadget : public gadget, private Ddraw
{
	friend class phtml_gadget;
	class phtml_gadget *p;

public:
	void GSetLimits(void);
	void GExpose(XEvent *);
	int  GActivate(XEvent *,int why);
	int  GSelected(XEvent *,int why,int wasabove);
	void GActionPress(int mx,int my,int rx,int ry,unsigned int state,int button,int reason);
	void GActionRelease(int mx,int my,int rx,int ry,unsigned int state,int button,int reason);
	int GMouseMove(int mx,int my,int rx,int ry,unsigned int state,int button);


	html_gadget(void);
	~html_gadget(void);

	int Create(void);
	void Free(void);

	void Resize(int w,int h);
	void Lock(void);
	void Unlock(void);
	void Block(int b);
	void Unblock(int b);

	void MinCols(int);
	void MinRows(int);

	void Text(char *html);
};

#endif
