#ifndef INCL_LEFTTEXT_H
#define INCL_LEFTTEXT_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/types.h>
#include <Xclasses/gadget.h>
#include <Xclasses/Xcolors.h>

class lefttext : public gadget 
{
	friend class plefttext;
	
	class plefttext *p;

public:
    virtual void GExpose(XEvent *event);
	virtual void GSetLimits(void);

public:
    lefttext(void);
    lefttext(char *);
    lefttext(char *,gadget &);
    lefttext(char *,gadget *);
    lefttext(gadget &lgg,gadget &gg);
    lefttext(gadget *lgg,gadget *gg);
    virtual ~lefttext();

    int Create(void);
    void Free(void);
    void Lock(void);
    void Unlock(void);
    void Block(int);
    void Unblock(int);
    void Resize(int w,int h);
	void Text(char *t);
	void Text(gadget *g) { LeftGadget(g); };
	void Text(gadget &g) { Text (&g); };
	void Top(void);
	void Middle(void);
	void Bottom(void);
	void Gadget(gadget *);
	void Gadget(gadget &g) { Gadget(&g); };
	void LeftGadget(gadget *);
	void LeftGadget(gadget &g) { LeftGadget(&g); };
};

#define HAS_LEFTGAGDET

#endif
