#ifndef INCL_LISTVIEW_H
#define INCL_LISTVIEW_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/types.h>
#include <Xclasses/multilistview.h>
#include <Xclasses/Xcolors.h>
#include <Xclasses/Ddraw.h>
#include <Xclasses/callable.h>

class listview : public multilistview
{
public:
    listview();
    virtual ~listview();

 	void SetHook(int (*hook)(struct hook_data*),void *userdata=NULL);
 	void SetHook(callable *c,int (callable::*hook)(struct hook_data*),void *userdata=NULL);

// item management
	void AddAtEnd(char *);
	void AddAtBegin(char *);
	void AddBefore(int,char *);

	void AddAtEnd(char *,void *data);
	void AddAtBegin(char *,void *data);
	void AddBefore(int,char *,void *data);

	void Replace(int,char *);
};

#define SETHOOK int (callable::*)(struct hook_data*)

#endif
