#ifndef CODE_VERSION_CXX
#define CODE_VERSION_CXX

/*-----------------------------------------------------------------------
 * Xclasses code file
 * ($Id: version.cxx,v 1.1.1.1 1998/07/16 19:29:27 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include "include/types.h"
#include "version.h"

int LibMajor(void)
{
	return LIBMAJOR;
}
int LibMinor(void)
{
	return LIBMINOR;
}
int LibPatch(void)
{
	return LIBPATCH;
}
int LibCompile(void)
{
	return LIBCOMPILE;
}
char *LibVersion(void)
{
	return LIBVERSION;
}
char *LibDate(void)
{
	return __DATE__;
}

#endif
