#ifndef CODE_SPACEGROUP_CXX
#define CODE_SPACEGROUP_CXX

/*-----------------------------------------------------------------------
 * Xclasses code file
 * ($Id: spacegroup.cxx,v 1.1.1.1 1998/07/16 19:29:28 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include "types.h"
#include <Xclasses/gadget.h>
#include <Xclasses/Xdisplay.h>
#include <Xclasses/spacegroup.h>
#include "include/spacegroup_p.h"

spacegroup::spacegroup(void) : group()
{
	while (!(p=new pspacegroup))
	{
		outOfMemory("spacegroup");
	}
	p->o=this;
	Name("spacegroup");
}
pspacegroup::pspacegroup()
{
	border=0;
}
spacegroup::~spacegroup(void)
{
	if (p) delete p;
}
pspacegroup::~pspacegroup()
{
}


void spacegroup::Border(int b)
{
	p->border=b;
}

// überladen
void spacegroup::BorderLimits(void)
{
	borderleft=p->border;
	borderright=p->border;
	borderbottom=p->border;
	bordertop=p->border;
}

#endif
