#ifndef XCLASSES_DEFAULTS
#define XCLASSES_DEFAULTS

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: defaults.h,v 1.3 1998/12/16 17:48:22 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#define DEFAULTFONT "-*-*-*-*-*-*-10-*-*-*-*-*-*-*"

// 1 - gadget
// 2 - group
// 3 - fixed
// 4 - small
// 5 - big
// 6 - menu
// 7 - help

typedef enum {normal,irix,pOS} embosstype;
typedef enum {shift,control,alt} listmultikey;
typedef enum {middle,right,none} listmultibutton;

struct defaultstruct
{
	char 			basecolor[20];
	char 			font[8][80];
	int  			nopopupmove;
	int  			help_wait;
	int  			help_shown;
	int				help_bubblemode;
	embosstype		scroller_emboss;
	embosstype		slider_emboss;
	int				scroller_buttonstogether;
	int				dclickdelay;
	listmultikey	listview_multikey;
	listmultikey	listview_multiblockkey;
	listmultibutton	listview_multibutton;
	int				menunewstyle;
	int				popupline;
	int				popuplight;
	int				writeresbase;
	int				intlkey;
	int				new3D;
	int				inputmode;
};

extern struct defaultstruct defaults;

#define LOAD_ALL_XCLASSES_CONFIG	"load all Xclasses config"

#ifndef FONT_HELP
#define FONT_HELP	7
#endif

#endif
