#ifndef INCL_GFXTEXT_H
#define INCL_GFXTEXT_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/types.h>
#include <Xclasses/gfx_object.h>

class gfx_text : public gfx_object
{
	friend class pgfx_text;
	class pgfx_text *p;
public:

	gfx_text(void);
	gfx_text(char *);
	virtual ~gfx_text();
	
	virtual void Adjust(int);

	virtual void draw_normal(class gadget *g,int x,int y,int w,int h);
	virtual void draw_selected(class gadget *g,int x,int y,int w,int h);
	virtual void draw_locked(class gadget *g,int x,int y,int w,int h);
	virtual void draw_selectedlocked(class gadget *g,int x,int y,int w,int h);

	virtual int height(class gadget *g);
	virtual int width(class gadget *g);

	void Text(char *);
	void NoUnderlineText(char *txt);	// ignores underline as ShortCut mark
	char ShortCut(void);
	static char ShortCut(char *);
};

#endif
