#ifndef INCL_ABOUTDIALOG_H
#define INCL_ABOUTDIALOG_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: about_dialog.h,v 1.1.1.1 1998/07/16 19:29:29 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/types.h>
#include <Xclasses/Xwindows.h>
#include <Xclasses/Xcolors.h>


class about_dialog : public Xcolors
{
	friend class pabout_dialog;
	
	class pabout_dialog *p;

public:
    about_dialog(int flag=0);
    virtual ~about_dialog();

	int Start(Xwindows *w);
	int Start(Xwindows &w) { return Start(&w); };
};

#endif
