#ifndef CODE_XCLOCKP_H
#define CODE_XCLOCKP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: xclock_p.h,v 1.4 2001/01/02 23:22:07 schmitzj Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <time.h>
#include <Xclasses/Xdisplay.h>


class pxclock
{
	friend class xclock;
	
public:
	class xclock *o;

	time_t ot;
	int hx1,hx2,hy1,hy2;
	int mx1,mx2,my1,my2;
	int sx1,sx2,sy1,sy2;
	int lw;

	callback	tcb;
	
	int seconds:1,
	    exact:1,
	    nominutes:1,
	    small:1;
	int form;

    pxclock();
    ~pxclock();

	void zeiger(time_t);
	void ziffern(void);
	int X(int,int);
	int Y(int,int);
};

#endif
