#ifndef CODE_XSHADOWP_H
#define CODE_XSHADOWP_H

/*-----------------------------------------------------------------------
 * Xclasses header file
 * ($Id: Xshadow_p.h,v 1.1.1.1 1998/07/16 19:29:32 cvs Exp $)
 * 
 * Copyright (C) 1998 Jürgen Schmitz
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *-----------------------------------------------------------------------
 */

#include <Xclasses/Xshadow.h>
#include <Xclasses/gadget.h>

class shadowgadget : public gadget
{
public:

	int Create(void)
	{
		if (gadget::Create())
		{
			Mode(GM_ONEEXPOSE);
			return 1;
		}
		return 0;
	}
	void GExpose(XEvent *e)
	{
		XSetForeground(display(),gc,col_black());
		XFillRectangle(display(),win,gc,0,0,width,height);
	}
};
class pXshadow
{
	friend class Xshadow;

public:
	class Xshadow *o;

	Pixmap 			shape_map;
	GC 				shape_gc;
	shadowgadget 	shape_gg;
	GC 				win_gc;
	Xwindows		*mywin;

	pXshadow();
	~pXshadow();

	void shadow(void);
	void resize(void);
};

#endif
